from django.core.management.base import BaseCommand, CommandError
from arkindex.images.models import ImageServer
from arkindex.documents.models import Volume
from arkindex.images.importer import import_indexes
from arkindex.documents.importer import import_page
import os
import logging
import csv

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)s] %(message)s',
)
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Import images & documents from a unique CSV file'
    volumes = {}  # cache

    def add_arguments(self, parser):
        parser.add_argument(
            'csv_path',
            help='Unique generated file'
        )
        parser.add_argument(
            '--iiif-server',
            type=str,
            help='IIIF server where to find images',
            required=True,
        )
        parser.add_argument(
            '--index-directory',
            type=str,
            help='Directory containing the index .GZ files',
            required=False,
        )

    def handle(self, *args, **options):
        assert os.path.exists(options['csv_path']), \
            'Invalid CSV path {}'.format(options['csv_path'])

        # Load the server
        try:
            server_id = options['iiif_server']
            if server_id.isdigit():
                server = ImageServer.objects.get(pk=server_id)
            else:
                server = ImageServer.objects.get(name=server_id)
        except Exception as e:
            raise CommandError("Image server not found: {}".format(e))

        logger.info('Using Server: {}'.format(server))

        imported = 0
        with open(options['csv_path'], 'r') as f:
            for line in csv.DictReader(f):
                try:
                    self.load(server, options['index_directory'], **line)
                    imported += 1
                except Exception as e:
                    logger.error(e)

        logger.info('Imported {} images + metadata'.format(imported))

    def load(self, server, index_directory, image, index, register, folio, **kwargs):
        '''
        Import image+indexes+metadata
        '''
        # Load image
        image_name = image  # backup for exception
        image = server.find_image(image_name)
        assert image is not None, \
            'Image not found {}'.format(image_name)
        logger.info('Image {}'.format(image))

        # Apply indexes
        index_path = os.path.realpath(os.path.join(
            index_directory or '',
            index,
        ))
        import_indexes(image, index_path)

        # Cache volume
        if register in self.volumes:
            volume = self.volumes[register]
        else:
            volume, _ = Volume.objects.get_or_create(name=register)
            self.volumes[register] = volume

        # Apply metadata
        page, created = import_page(volume, image, register, folio)
        logger.info('{} {}'.format(created and 'Created' or 'Updated', page))
